# Chat text mode

#    The Circle - Decentralized resource discovery software
#    Copyright (C) 2001  Paul Francis Harrison
#    Copyright (C) 2002  Monash University
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from __future__ import generators

import os
import select
import string
import sys
import utility
import error
from chat import *


class Chat_field_text_group:
    
    def __init__(self, chat):
        self.chat = chat
        self.members = [ ]
        self.n_open = 0

    def add(self, member,after=None):
        if after:
            self.members.insert(self.members.index(after)+1,member)
        else:
            self.members.append(member)
        self.n_open = self.n_open + 1

    def closure(self):
        self.n_open = self.n_open - 1

        if not self.n_open:
            self.chat.text_hide_command_line()
            for member in self.members:
                self.chat.output.write(utility.force_string(member.text))
            self.chat.output.flush()
            self.chat.text_show_command_line()
            self.chat = None



class Chat_field_text:
    def __init__(self, chat=None,group=None,after=None):
        if not group:
            group = Chat_field_text_group(chat)
        self.text = ''
        self.group = group
        group.add(self,after)

        
    def child(self,action=None):
        # Only function that can be called from other thread
        return Chat_field_text(group=self.group,after=self)

    def show(self, str, dummy_tags=[], dummy_augmentation=None):
        self.text = str

    def close(self):
        self.group.closure()

    def set_action(self,action=None):
        pass



class Chat_text(Chat):
    """ Text mode chat interface. """
    
    def __init__(self,app):
        Chat.__init__(self,app,0)

        self.prompt = ''
        self.command_text_lines = [ ]
        self.command_text = ''
        self.input_queue = ''
        self.escape_queue = [ ]
        self.command_visible = 0

        self.input = app.input
        self.output = app.output
        self.widget_visible = 1

        self.shell = None

    def stop(self):
        # main thread only!
        utility.stdin_echo(0)
        self.text_hide_command_line()
        utility.stdin_echo(1)

        Chat.stop(self)

    def run_shell(self, command):
        self.text_hide_command_line()
        utility.stdin_echo(1)
        os.system(command)
        utility.stdin_echo(0)
        self.text_show_command_line()

    def set_prompt(self, str):
        self.lock.acquire()
        try:
            if str == self.prompt:
                return
            
            self.text_hide_command_line()
            self.prompt = str
            self.text_show_command_line()
        finally:
            self.lock.release()
        return

    def get_field(self,action=None):
        return Chat_field_text(self)

    def get_field_before(self,field,action=None):
        return Chat_field_text(self)

    def get_field_after(self,field,action=None):
        return Chat_field_text(self)
    
    def set_tabs(self,length):
        return

    def text_read_input(self):
        char = self.input.read(1)
        if not char:
            raise error.Error('interrupted')
        
        self.input_queue = self.input_queue + char

        if (char >= 'a' and char <= 'z') or \
             (char >= 'A' and char <= 'Z'):
            pos = string.find(self.input_queue,chr(27)+'[')
            if pos != -1:
                self.escape_queue.append(self.input_queue[pos+2:])
                self.input_queue = self.input_queue[:pos]

    def text_get_escape(self,code):
        while 1:
            for i in range(len(self.escape_queue)):
                if self.escape_queue[i][-1] == code:
                    return self.escape_queue.pop(i)
            self.text_read_input()

    def text_backspace(self, n):
        while n:
            self.output.write(chr(27)+'[6n')
            self.output.flush()

            escape = self.text_get_escape('R')

            try:
                column = int(string.split(escape[:-1],';')[1]) - 1
            except:
                print _('Badly formatted escape sequence '),escape
                return

            if column == 0:
                self.output.write(chr(27)+'[A'+chr(27)+'[999C'+chr(27)+'[K')
                self.output.flush()
                n = n - 1
                continue
            
            if n < column:
                move = n
            else:
                move = column
            self.output.write(chr(27)+'[%dD'%move+chr(27)+'[K')
            self.output.flush()

            n = n - move

    def text_hide_command_line(self):
        #sys.stdout.write( (chr(8)+' '+chr(8))*(len(self.prompt)+len(self.command_text)) )
        #sys.stdout.flush()
        self.command_visible = self.command_visible - 1
        if self.command_visible == 0:
            self.text_backspace( len(self.prompt)+len(self.command_text)+1 )

    def text_show_command_line(self):
        self.command_visible = self.command_visible + 1
        if self.command_visible == 1:
            self.output.write( self.prompt+' '+self.command_text )
            self.output.flush()

    def text_advance_command_line(self):
        self.output.write('\n')
        self.command_text = ''
        self.command_visible = 0

    #def text_mainloop(self):
    #  while 1:
    #    try:
    #      self.text_mainloop_inner()
#       return
#      except error.Error, err:
#        if err.message == 'sigint':
#         self.command_text = ''
#         self.command_text_lines = [ ]
#         self.set_normal_prompt()
#       else:
#         raise err

    def text_mainloop(self):
        self.command_text_lines = [ ]
        self.command_text = ''
        self.set_normal_prompt()

        utility.set_config('text_status',{'running':1})

        utility.stdin_echo(0)
        self.text_show_command_line()
        try:
            utility.threadnice_mainloop_start()
            while utility.threadnice_mainloop_is_running():
                utility.threadnice_mainloop_iteration()
                    
                if select.select([self.input],[],[ ],0.1)[0]:
                    self.text_read_input()

                while self.escape_queue:
                    item = self.escape_queue.pop(0)

                    if self.command_text_lines:
                        continue

                    if item == 'A' and self.history_position > 0:
                        self.history[self.history_position] = self.command_text
                        self.history_position = self.history_position - 1
                        self.text_hide_command_line()
                        self.command_text = self.history[self.history_position]
                        self.text_show_command_line()

                    if item == 'B' and self.history_position < len(self.history)-1:
                        self.history[self.history_position] = self.command_text
                        self.history_position = self.history_position + 1
                        self.text_hide_command_line()
                        self.command_text = self.history[self.history_position]
                        self.text_show_command_line()

                while self.input_queue and self.input_queue != chr(27) and self.input_queue[:2] != chr(27)+'[':
                    char = self.input_queue[0]
                    self.input_queue = self.input_queue[1:]

                    if char == '\n':
                        line = self.command_text
                        self.text_advance_command_line()
                            
                        self.history[-1] = line
                        self.history.append('')
                        self.history_position = len(self.history)-1

                        if line[-1:] == '\\':
                            self.command_text_lines.append(line[:-1])
                            self.set_normal_prompt()
                            self.text_show_command_line()
                        else:
                            lines = self.command_text_lines
                            self.command_text_lines = [ ]
                            lines.append(line)
                            command = string.strip(string.join(lines,'\n'))
                            self.handle_user_entry(command)
                            self.text_show_command_line()
                        continue

                    if char == chr(4):
                        self.text_hide_command_line()
                        self.show(_('Entering shell (Circle is still running)\n'))

                        self.lock.acquire()
                        old_quiet = self.quiet
                        old_activity = self.activity
                        self.lock.release()

                        if not old_quiet:
                            self.set_status(1,'(shell)')
        
                        command = 'exec $SHELL'
                        status = utility.get_config('text_status',{ })
                        if status.get('directory',None):
                            command = 'cd '+utility.quote_for_shell(status['directory'])+' && '+command
                            check_directory = 0
                        else:
                            check_directory = 1

                        utility.stdin_echo(1)
                        try:
                            self.do('//'+command)
                            self.show('\n')
                        finally:
                            utility.stdin_echo(0)
                            self.text_show_command_line()
                        
                        if check_directory:
                            status = utility.get_config('text_status',{ })
                            if not status.has_key('directory'):
                                self.show(_("Current working directory was not saved.\n"+\
                                                    "Suggest adding `circle -s` to your shell prompt (see /help).\n"))

                        if not old_quiet:
                            self.set_status(old_quiet,old_activity)

                        continue

                    if ord(char) < 32:
                        continue
                    
                    if char == chr(127):
                        if self.command_text:
                            self.command_text = self.command_text[:-1]
                            #sys.stdout.write(chr(8)+' '+chr(8))
                            #sys.stdout.flush()
                            self.text_backspace(1)
                    else:
                        self.output.write(char)
                        self.output.flush()
                        self.command_text = self.command_text + char

        finally:
            utility.set_config('text_status',{'running':0})
            utility.threadnice_mainloop_stop()
            self.text_hide_command_line()
            utility.stdin_echo(1)


    def gossip_field_on_click(self, field, widget, event):
        pass

    def tip_field_on_click(self, field, widget, event):
        pass

    def host_field_on_click(self, field, widget, event):
        pass

    def name_field_on_click(self,field,widget,event):
        pass

    def channel_field_on_click(self,field,widget,event):
        pass

    def url_field_on_click(self,field,widget,event):
        pass

    def file_field_on_click(self,field,widget,event):
        pass

    def directory_field_on_click(self,d_field,widget,event,root):
        pass

    def take_field_on_click(self,field,widget,event):
        pass

# vim: expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
