#    The Circle - Decentralized resource discovery software
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import utility
import gtk
import check
import sys

class Download_manager:
    """
    This is just the downloads window.
    I guess I do not need a class for that,
    a function that raises the window would do it
    """
    
    def __init__(self, app):
        import gtk

        self.app=app
        self.file_server = app.file_server

        self.window = gtk.Window()
        self.window.set_title("Download Manager")
        self.window.set_border_width(10)
        self.window.set_default_size(720,150)

        vbox = gtk.VBox(0,5)
        self.window.add(vbox)
        vbox.show()

        scrolly = gtk.ScrolledWindow()
        scrolly.set_policy(gtk.POLICY_NEVER,gtk.POLICY_AUTOMATIC)
        vbox.pack_start(scrolly, 1,1,0)
        scrolly.show()

        self.list = gtk.CList(2)
        self.list.set_column_width(0,300)
        self.list.set_selection_mode(gtk.SELECTION_SINGLE)
        scrolly.add(self.list)
        self.list.show()

        hbox = gtk.HBox(0,5)
        vbox.pack_start(hbox, 0,0,0)
        hbox.show()

        self.cancel_button = gtk.Button('Cancel download')
        hbox.pack_start(self.cancel_button, 1,1,0)

        self.play_button = gtk.Button('Play')
        hbox.pack_start(self.play_button, 1,1,0)

        def on_delete(window, event, self=self):
            self.assert_invisible()
            return 1
        self.window.connect("delete_event",on_delete)

        def on_select(list,index,column,event,self=self):
            self.selected_row = index
            self.cancel_button.show()
            self.play_button.show()
        self.list.connect("select-row",on_select)

        def on_unselect(list,index,column,event,self=self):
            self.cancel_button.hide()
            self.play_button.hide()
        self.list.connect("unselect-row",on_unselect)

        def on_cancel(button, self=self):
            downloader=self.file_server.downloaders[self.selected_row]
            downloader.cancel_download()
            downloader.stop()
            self.on_complete(self.selected_row)
        self.cancel_button.connect("clicked",on_cancel)

        def on_play(button, self=self):
            downloader=self.file_server.downloaders[self.selected_row]
            self.app.music_manager.append_song(downloader.filename,1,downloader.data['length'],downloader)
        self.play_button.connect("clicked",on_play)
        
        self.visible = 0
        
    def on_complete(self,row):
        self.list.remove(row)
        del self.file_server.downloaders[row]
        if len(self.file_server.downloaders) == 0:
            self.assert_invisible()

    def assert_invisible(self):
        """Only call from main thread"""
        check.check_is_gtkthread()
        if self.visible:
            self.window.hide()
            self.visible=0

    def assert_visible(self):
        """Only call from main thread"""
        check.check_is_gtkthread()
        if not self.visible:
            self.app.show_window(self.window, 'download', show_all=0)

            # Obscure GTK bug workaround
            # thomasV : I think this is useless, at least on linux ... added the test
            if sys.platform != 'linux2':
                import gtk
                while gtk.events_pending():
                    gtk.mainiteration()
                
            self.visible = 1

    def start(self):
        #Multiple calls to timeout_add crash windows gtk
        gtk.timeout_add(100,self.update)
        self.update()

    def stop(self):
        #self.assert_invisible()
        self.window.destroy()

    def complete_last(self):
        i = len(self.file_server.downloaders) - 1
        del self.file_server.downloaders[i]
        self.list.remove(i)
    

    def update(self):

        check.check_is_gtkthread()
        
        if not self.app.running :
            return 0

        for i in range(len(self.file_server.downloaders)):
            downloader= self.file_server.downloaders[i]
            if self.list.rows <i+1:
                self.list.append([downloader.basename,''])
            
            if not downloader.running:
                for field in downloader.fields:
                    if downloader.success:
                        field.show(_('done.\n'),['grey'])
                    else:
                        field.show(_('canceled.\n'),['grey'])
                    field.close()
                self.on_complete(i)
                break
            else:
                str = ((" %d%% " % ((downloader.get_bytes_downloaded()+1)*100
                                    / (downloader.get_bytes_total()+1)))
                       + 'of ' + utility.human_size(downloader.get_bytes_total())
                       + '  ' + utility.human_size(downloader.get_speed()) + '/s'
                       + ' '  + downloader.get_remaining_time()
                       + '  (' + downloader.get_sources()) +')'
                self.list.set_text(i,1,str)

                str= ("%d%% " % ((downloader.get_bytes_downloaded()+1)*100/ (downloader.get_bytes_total()+1)))
                if not downloader.authorized_links:
                    str = str +' '+ downloader.get_sources()
                else:
                    str = str + ' ' + utility.human_size(downloader.get_speed()) + '/s'\
                          + '\t'  + downloader.get_remaining_time()

                for field in downloader.fields:
                    field.show(str+'\n',['grey'])

        return 1
        

